@@ -18,9 +18,9 @@ package org.gradle.api.file;
 
 import org.gradle.api.Incubating;
 import org.gradle.api.model.ObjectFactory;
-import org.gradle.api.provider.Property;
 import org.gradle.api.provider.Provider;
 
+import javax.annotation.Nullable;
 import java.io.File;
 
 /**
@@ -34,28 +34,35 @@ import java.io.File;
  *
  * @since 4.3
  */
-@Incubating
-public interface DirectoryProperty extends Property<Directory> {
-    /**
-     * Views the location of this directory as a {@link File}.
-     */
-    Provider<File> getAsFile();
-
+public interface DirectoryProperty extends FileSystemLocationProperty<Directory> {
     /**
      * Returns a {@link FileTree} that allows the files and directories contained in this directory to be queried.
      */
     FileTree getAsFileTree();
 
     /**
-     * Sets the location of this directory.
+     * {@inheritDoc}
      */
-    void set(File dir);
+    @Override
+    DirectoryProperty value(@Nullable Directory value);
 
     /**
      * {@inheritDoc}
      */
     @Override
-    DirectoryProperty value(Directory value);
+    DirectoryProperty value(Provider<? extends Directory> provider);
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    DirectoryProperty fileValue(@Nullable File file);
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    DirectoryProperty fileProvider(Provider<File> provider);
 
     /**
      * {@inheritDoc}
@@ -100,4 +107,18 @@ public interface DirectoryProperty extends Property<Directory> {
      * @return The file.
      */
     Provider<RegularFile> file(Provider<? extends CharSequence> path);
+
+    /**
+     * Returns a {@link FileCollection} containing the given files,
+     * whose locations are the given paths resolved relative to this directory,
+     * as defined by {@link org.gradle.api.Project#files(Object...)}.
+     *
+     * This method can also be used to create an empty collection, but the collection may not be mutated later.
+     *
+     * @param paths The paths to the files. May be empty.
+     * @return The file collection.
+     * @since 6.0
+     */
+    @Incubating
+    FileCollection files(Object... paths);
 }
