@@ -26,6 +26,7 @@ import org.gradle.api.internal.collections.ListElementSource;
 import org.gradle.api.specs.Spec;
 import org.gradle.api.specs.Specs;
 import org.gradle.internal.reflect.Instantiator;
+import org.gradle.util.DeprecationLogger;
 
 import java.util.ArrayList;
 import java.util.Collection;
@@ -41,8 +42,10 @@ public class DefaultNamedDomainObjectList<T> extends DefaultNamedDomainObjectCol
     /**
      * only left here to not break `nebula.dependency-recommender` plugin.
      */
+    @Deprecated
     public DefaultNamedDomainObjectList(Class<T> type, Instantiator instantiator, Namer<? super T> namer) {
         super(type, new ListElementSource<T>(), instantiator, namer, CollectionCallbackActionDecorator.NOOP);
+        DeprecationLogger.nagUserOfDeprecated("Internal API constructor DefaultNamedDomainObjectList(Class<T>, Instantiator, Namer<T>)", "Please use ObjectFactory.namedDomainObjectList(Class<T>) instead.");
     }
 
     public DefaultNamedDomainObjectList(Class<T> type, Instantiator instantiator, Namer<? super T> namer, CollectionCallbackActionDecorator decorator) {
