@@ -15,7 +15,11 @@
  */
 package org.gradle.api.tasks;
 
+import org.gradle.api.Incubating;
 import org.gradle.api.internal.ConventionTask;
+import org.gradle.api.model.ObjectFactory;
+import org.gradle.api.provider.Property;
+import org.gradle.api.provider.Provider;
 import org.gradle.process.CommandLineArgumentProvider;
 import org.gradle.process.ExecResult;
 import org.gradle.process.ExecSpec;
@@ -23,6 +27,7 @@ import org.gradle.process.ProcessForkOptions;
 import org.gradle.process.internal.ExecAction;
 import org.gradle.process.internal.ExecActionFactory;
 
+import javax.annotation.Nullable;
 import javax.inject.Inject;
 import java.io.File;
 import java.io.InputStream;
@@ -37,14 +42,20 @@ import java.util.Map;
  */
 public abstract class AbstractExecTask<T extends AbstractExecTask> extends ConventionTask implements ExecSpec {
     private final Class<T> taskType;
+    private final Property<ExecResult> execResult;
     private ExecAction execAction;
-    private ExecResult execResult;
 
     public AbstractExecTask(Class<T> taskType) {
         execAction = getExecActionFactory().newExecAction();
+        execResult = getObjectFactory().property(ExecResult.class);
         this.taskType = taskType;
     }
 
+    @Inject
+    protected ObjectFactory getObjectFactory() {
+        throw new UnsupportedOperationException();
+    }
+
     @Inject
     protected ExecActionFactory getExecActionFactory() {
         throw new UnsupportedOperationException();
@@ -52,7 +63,7 @@ public abstract class AbstractExecTask<T extends AbstractExecTask> extends Conve
 
     @TaskAction
     protected void exec() {
-        execResult = execAction.execute();
+        execResult.set(execAction.execute());
     }
 
     /**
@@ -104,7 +115,7 @@ public abstract class AbstractExecTask<T extends AbstractExecTask> extends Conve
      * {@inheritDoc}
      */
     @Override
-    public T setArgs(Iterable<?> arguments) {
+    public T setArgs(@Nullable Iterable<?> arguments) {
         execAction.setArgs(arguments);
         return taskType.cast(this);
     }
@@ -112,7 +123,9 @@ public abstract class AbstractExecTask<T extends AbstractExecTask> extends Conve
     /**
      * {@inheritDoc}
      */
-    @Optional @Input
+    @Nullable
+    @Optional
+    @Input
     @Override
     public List<String> getArgs() {
         return execAction.getArgs();
@@ -163,7 +176,9 @@ public abstract class AbstractExecTask<T extends AbstractExecTask> extends Conve
     /**
      * {@inheritDoc}
      */
-    @Optional @Input
+    @Nullable
+    @Optional
+    @Input
     @Override
     public String getExecutable() {
         return execAction.getExecutable();
@@ -173,7 +188,7 @@ public abstract class AbstractExecTask<T extends AbstractExecTask> extends Conve
      * {@inheritDoc}
      */
     @Override
-    public void setExecutable(String executable) {
+    public void setExecutable(@Nullable String executable) {
         execAction.setExecutable(executable);
     }
 
@@ -197,6 +212,7 @@ public abstract class AbstractExecTask<T extends AbstractExecTask> extends Conve
     /**
      * {@inheritDoc}
      */
+    @Override
     @Internal
     // TODO:LPTR Should be a content-less @InputDirectory
     public File getWorkingDir() {
@@ -352,9 +368,26 @@ public abstract class AbstractExecTask<T extends AbstractExecTask> extends Conve
      * Returns the result for the command run by this task. Returns {@code null} if this task has not been executed yet.
      *
      * @return The result. Returns {@code null} if this task has not been executed yet.
+     *
+     * @see #getExecutionResult() for the preferred way of accessing this property.
      */
     @Internal
+    @Nullable
     public ExecResult getExecResult() {
+        // TODO: Once getExecutionResult is stable, make this deprecated
+        // DeprecationLogger.nagUserOfReplacedMethod("AbstractExecTask.getExecResult()", "AbstractExecTask.getExecutionResult()");
+        return execResult.getOrNull();
+    }
+
+    /**
+     * Returns the result for the command run by this task. The provider has no value if this task has not been executed yet.
+     *
+     * @return A provider of the result.
+     * @since 6.1
+     */
+    @Internal
+    @Incubating
+    public Provider<ExecResult> getExecutionResult() {
         return execResult;
     }
 }
