@@ -30,6 +30,7 @@ import org.gradle.api.artifacts.ivy.IvyModuleDescriptor;
 import org.gradle.api.attributes.AttributeContainer;
 import org.gradle.api.internal.artifacts.ComponentMetadataProcessor;
 import org.gradle.api.internal.artifacts.MetadataResolutionContext;
+import org.gradle.api.internal.artifacts.dsl.dependencies.PlatformSupport;
 import org.gradle.api.internal.artifacts.ivyservice.DefaultIvyModuleDescriptor;
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.UserProvidedMetadata;
 import org.gradle.api.internal.artifacts.repositories.resolver.ComponentMetadataDetailsAdapter;
@@ -66,8 +67,6 @@ import java.util.ArrayList;
 import java.util.Collection;
 import java.util.List;
 
-import static org.gradle.api.internal.artifacts.repositories.resolver.VirtualComponentHelper.makeVirtual;
-
 public class DefaultComponentMetadataProcessor implements ComponentMetadataProcessor {
 
     private final static boolean FORCE_REALIZE = Boolean.getBoolean("org.gradle.integtest.force.realize.metadata");
@@ -134,6 +133,7 @@ public class DefaultComponentMetadataProcessor implements ComponentMetadataProce
     private final ComponentMetadataRuleExecutor ruleExecutor;
     private final MetadataResolutionContext metadataResolutionContext;
     private final ComponentMetadataRuleContainer metadataRuleContainer;
+    private final PlatformSupport platformSupport;
 
     public DefaultComponentMetadataProcessor(ComponentMetadataRuleContainer metadataRuleContainer,
                                              Instantiator instantiator,
@@ -142,6 +142,7 @@ public class DefaultComponentMetadataProcessor implements ComponentMetadataProce
                                              NotationParser<Object, ComponentIdentifier> componentIdentifierNotationParser,
                                              ImmutableAttributesFactory attributesFactory,
                                              ComponentMetadataRuleExecutor ruleExecutor,
+                                             PlatformSupport platformSupport,
                                              MetadataResolutionContext resolutionContext) {
         this.metadataRuleContainer = metadataRuleContainer;
         this.instantiator = instantiator;
@@ -150,6 +151,7 @@ public class DefaultComponentMetadataProcessor implements ComponentMetadataProce
         this.componentIdentifierNotationParser = componentIdentifierNotationParser;
         this.attributesFactory = attributesFactory;
         this.ruleExecutor = ruleExecutor;
+        this.platformSupport = platformSupport;
         this.metadataResolutionContext = resolutionContext;
     }
 
@@ -181,13 +183,13 @@ public class DefaultComponentMetadataProcessor implements ComponentMetadataProce
         }
 
         if (!updatedMetadata.getStatusScheme().contains(updatedMetadata.getStatus())) {
-            throw new ModuleVersionResolveException(updatedMetadata.getModuleVersionId(), String.format("Unexpected status '%s' specified for %s. Expected one of: %s", updatedMetadata.getStatus(), updatedMetadata.getId().getDisplayName(), updatedMetadata.getStatusScheme()));
+            throw new ModuleVersionResolveException(updatedMetadata.getModuleVersionId(), () -> String.format("Unexpected status '%s' specified for %s. Expected one of: %s", updatedMetadata.getStatus(), updatedMetadata.getId().getDisplayName(), updatedMetadata.getStatusScheme()));
         }
         return updatedMetadata;
     }
 
     protected ComponentMetadataDetails createDetails(MutableModuleComponentResolveMetadata mutableMetadata) {
-        return instantiator.newInstance(ComponentMetadataDetailsAdapter.class, mutableMetadata, instantiator, dependencyMetadataNotationParser, dependencyConstraintMetadataNotationParser, componentIdentifierNotationParser);
+        return instantiator.newInstance(ComponentMetadataDetailsAdapter.class, mutableMetadata, instantiator, dependencyMetadataNotationParser, dependencyConstraintMetadataNotationParser, componentIdentifierNotationParser, platformSupport);
     }
 
     @Override
@@ -201,7 +203,7 @@ public class DefaultComponentMetadataProcessor implements ComponentMetadataProce
             updatedMetadata = details.asImmutable();
         }
         if (!updatedMetadata.getStatusScheme().contains(updatedMetadata.getStatus())) {
-            throw new ModuleVersionResolveException(updatedMetadata.getId(), String.format("Unexpected status '%s' specified for %s. Expected one of: %s", updatedMetadata.getStatus(), updatedMetadata.getId().toString(), updatedMetadata.getStatusScheme()));
+            throw new ModuleVersionResolveException(updatedMetadata.getId(), () -> String.format("Unexpected status '%s' specified for %s. Expected one of: %s", updatedMetadata.getStatus(), updatedMetadata.getId().toString(), updatedMetadata.getStatusScheme()));
         }
         return updatedMetadata;
     }
@@ -215,7 +217,7 @@ public class DefaultComponentMetadataProcessor implements ComponentMetadataProce
         for (MetadataRuleWrapper wrapper : metadataRuleContainer) {
             if (wrapper.isClassBased()) {
                 Collection<SpecConfigurableRule> rules = wrapper.getClassRules();
-                Action<ComponentMetadataContext> action = collectRulesAndCreateAction(rules, id, instantiator);
+                Action<ComponentMetadataContext> action = collectRulesAndCreateAction(rules, id, metadataResolutionContext.getInjectingInstantiator());
                 processClassRule(action, metadata, details);
             } else {
                 processRule(wrapper.getRule(), metadata, details);
@@ -240,7 +242,7 @@ public class DefaultComponentMetadataProcessor implements ComponentMetadataProce
                 new Transformer<WrappingComponentMetadataContext, ModuleComponentResolveMetadata>() {
                     @Override
                     public WrappingComponentMetadataContext transform(ModuleComponentResolveMetadata moduleVersionIdentifier) {
-                        return new WrappingComponentMetadataContext(metadata, instantiator, dependencyMetadataNotationParser, dependencyConstraintMetadataNotationParser, componentIdentifierNotationParser);
+                        return new WrappingComponentMetadataContext(metadata, instantiator, dependencyMetadataNotationParser, dependencyConstraintMetadataNotationParser, componentIdentifierNotationParser, platformSupport);
                     }
                 }, metadataResolutionContext.getCachePolicy());
         } catch (InvalidUserCodeException e) {
@@ -312,7 +314,6 @@ public class DefaultComponentMetadataProcessor implements ComponentMetadataProce
         private boolean changing;
         private List<String> statusScheme;
         private AttributeContainerInternal attributes;
-        private final List<ComponentIdentifier> owners;
 
         public ShallowComponentMetadataAdapter(NotationParser<Object, ComponentIdentifier> componentIdentifierNotationParser, ComponentMetadata source, ImmutableAttributesFactory attributesFactory) {
             this.componentIdentifierNotationParser = componentIdentifierNotationParser;
@@ -320,7 +321,6 @@ public class DefaultComponentMetadataProcessor implements ComponentMetadataProce
             changing = source.isChanging();
             statusScheme = source.getStatusScheme();
             attributes = attributesFactory.mutable((AttributeContainerInternal) source.getAttributes());
-            owners = Lists.newArrayListWithExpectedSize(1);
         }
 
         @Override
@@ -348,18 +348,29 @@ public class DefaultComponentMetadataProcessor implements ComponentMetadataProce
 
         }
 
+        @Override
+        public void addVariant(String name, Action<? super VariantMetadata> action) {
+
+        }
+
+        @Override
+        public void addVariant(String name, String base, Action<? super VariantMetadata> action) {
+
+        }
+
+        @Override
+        public void maybeAddVariant(String name, String base, Action<? super VariantMetadata> action) {
+
+        }
+
         @Override
         public void belongsTo(Object notation) {
-            belongsTo(notation, true);
+
         }
 
         @Override
         public void belongsTo(Object notation, boolean virtual) {
-            ComponentIdentifier id = componentIdentifierNotationParser.parseNotation(notation);
-            if (virtual) {
-                id = makeVirtual(id);
-            }
-            owners.add(id);
+
         }
 
         @Override
