@@ -71,6 +71,9 @@ public class DefaultResolutionStrategy implements ResolutionStrategyInternal {
     private boolean dependencyLockingEnabled = false;
     private boolean assumeFluidDependencies;
     private SortOrder sortOrder = SortOrder.DEFAULT;
+    private boolean failOnDynamicVersions;
+    private boolean failOnChangingVersions;
+
     private static final String ASSUME_FLUID_DEPENDENCIES = "org.gradle.resolution.assumeFluidDependencies";
 
     public DefaultResolutionStrategy(DependencySubstitutionRules globalDependencySubstitutionRules,
@@ -120,6 +123,27 @@ public class DefaultResolutionStrategy implements ResolutionStrategyInternal {
         return this;
     }
 
+    @Override
+    public ResolutionStrategy failOnDynamicVersions() {
+        mutationValidator.validateMutation(STRATEGY);
+        this.failOnDynamicVersions = true;
+        return this;
+    }
+
+    @Override
+    public ResolutionStrategy failOnChangingVersions() {
+        mutationValidator.validateMutation(STRATEGY);
+        this.failOnChangingVersions = true;
+        return this;
+    }
+
+    @Override
+    public ResolutionStrategy failOnNonReproducibleResolution() {
+        failOnChangingVersions();
+        failOnDynamicVersions();
+        return this;
+    }
+
     @Override
     public void preferProjectModules() {
         conflictResolution = ConflictResolution.preferProjectModules;
@@ -132,6 +156,14 @@ public class DefaultResolutionStrategy implements ResolutionStrategyInternal {
         return this;
     }
 
+    @Override
+    public ResolutionStrategy deactivateDependencyLocking() {
+        mutationValidator.validateMutation(STRATEGY);
+        dependencyLockingEnabled = false;
+        return this;
+    }
+
+
     @Override
     public void sortArtifacts(SortOrder sortOrder) {
         this.sortOrder = sortOrder;
@@ -266,6 +298,12 @@ public class DefaultResolutionStrategy implements ResolutionStrategyInternal {
         if (isDependencyLockingEnabled()) {
             out.activateDependencyLocking();
         }
+        if (isFailingOnDynamicVersions()) {
+            out.failOnDynamicVersions();
+        }
+        if (isFailingOnChangingVersions()) {
+            out.failOnChangingVersions();
+        }
         return out;
     }
 
@@ -287,4 +325,14 @@ public class DefaultResolutionStrategy implements ResolutionStrategyInternal {
     public CapabilitiesResolutionInternal getCapabilitiesResolutionRules() {
         return capabilitiesResolution;
     }
+
+    @Override
+    public boolean isFailingOnDynamicVersions() {
+        return failOnDynamicVersions;
+    }
+
+    @Override
+    public boolean isFailingOnChangingVersions() {
+        return failOnChangingVersions;
+    }
 }
