@@ -77,6 +77,8 @@ class SelectorState implements DependencyGraphSelector, ResolvableSelectorState
     private boolean forced;
     private boolean softForced;
     private boolean fromLock;
+    private boolean reusable;
+    private boolean markedReusableAlready;
 
     // The following state needs to be tracked to consistently construct `ComponentOverrideMetadata` independent of the order dependencies are visited
     private IvyArtifactName firstDependencyArtifact;
@@ -211,6 +213,7 @@ class SelectorState implements DependencyGraphSelector, ResolvableSelectorState
     }
 
     private boolean requiresResolve(ComponentIdResolveResult previousResult, VersionSelector allRejects) {
+        this.reusable = false;
         // If we've never resolved, must resolve
         if (previousResult == null) {
             return true;
@@ -243,12 +246,45 @@ class SelectorState implements DependencyGraphSelector, ResolvableSelectorState
         return resolved;
     }
 
+    /**
+     * Marks a selector for reuse,
+     * indicating it could be used again for resolution
+     *
+     * @return {@code true} if that selector has been marked for reuse before, {@code false} otherwise
+     */
+    boolean markForReuse() {
+        if (!resolved) {
+            // Selector was marked for deferred selection - let's not trigger selection now
+            return true;
+        }
+        this.reusable = true;
+        if (markedReusableAlready) {
+            return true;
+        } else {
+            markedReusableAlready = true;
+            return false;
+        }
+    }
+
+    /**
+     * Checks if the selector can be used for resolution.
+     *
+     * @return {@code true} if the selector can resolve, {@code false} otherwise
+     */
+    boolean canResolve() {
+        if (reusable) {
+            return true;
+        }
+        return !resolved;
+    }
+
     /**
      * Overrides the component that is the chosen for this selector.
      * This happens when the `ModuleResolveState` is restarted, during conflict resolution or version range merging.
      */
     public void overrideSelection(ComponentState selected) {
         this.resolved = true;
+        this.reusable = false;
 
         // Target module can change, if this is called as the result of a module replacement conflict.
         this.targetModule = selected.getModule();
@@ -382,7 +418,7 @@ class SelectorState implements DependencyGraphSelector, ResolvableSelectorState
         changing = changing || dependencyState.getDependency().isChanging();
     }
 
-    private class UnmatchedVersionsReason implements Describable {
+    private static class UnmatchedVersionsReason implements Describable {
         private final Collection<String> rejectedVersions;
         private final ComponentSelectionDescriptorInternal descriptor;
 
