@@ -19,20 +19,20 @@ import com.google.common.collect.ImmutableList;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Sets;
 import org.gradle.api.Action;
-import org.gradle.api.InvalidUserDataException;
 import org.gradle.api.Transformer;
 import org.gradle.api.artifacts.ComponentMetadataListerDetails;
 import org.gradle.api.artifacts.ComponentMetadataSupplierDetails;
+import org.gradle.api.artifacts.ModuleIdentifier;
+import org.gradle.api.artifacts.component.ModuleComponentIdentifier;
 import org.gradle.api.artifacts.repositories.AuthenticationContainer;
 import org.gradle.api.artifacts.repositories.MavenArtifactRepository;
 import org.gradle.api.artifacts.repositories.MavenRepositoryContentDescriptor;
-import org.gradle.api.internal.FeaturePreviews;
-import org.gradle.internal.instantiation.InstantiatorFactory;
-import org.gradle.api.internal.artifacts.ImmutableModuleIdentifierFactory;
 import org.gradle.api.internal.artifacts.ModuleVersionPublisher;
+import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.ComponentResolvers;
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.ConfiguredModuleComponentRepository;
+import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.parser.GradleModuleMetadataParser;
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.parser.MetaDataParser;
-import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.parser.ModuleMetadataParser;
+import org.gradle.internal.hash.ChecksumService;
 import org.gradle.api.internal.artifacts.repositories.descriptor.MavenRepositoryDescriptor;
 import org.gradle.api.internal.artifacts.repositories.descriptor.RepositoryDescriptor;
 import org.gradle.api.internal.artifacts.repositories.maven.MavenMetadataLoader;
@@ -44,8 +44,11 @@ import org.gradle.api.internal.artifacts.repositories.metadata.ImmutableMetadata
 import org.gradle.api.internal.artifacts.repositories.metadata.MavenMetadataArtifactProvider;
 import org.gradle.api.internal.artifacts.repositories.metadata.MavenMutableModuleMetadataFactory;
 import org.gradle.api.internal.artifacts.repositories.metadata.MetadataSource;
+import org.gradle.api.internal.artifacts.repositories.metadata.RedirectingGradleMetadataModuleMetadataSource;
 import org.gradle.api.internal.artifacts.repositories.resolver.ExternalResourceArtifactResolver;
 import org.gradle.api.internal.artifacts.repositories.resolver.MavenResolver;
+import org.gradle.api.internal.artifacts.repositories.resolver.ResourcePattern;
+import org.gradle.api.internal.artifacts.repositories.resolver.VersionLister;
 import org.gradle.api.internal.artifacts.repositories.transport.RepositoryTransport;
 import org.gradle.api.internal.artifacts.repositories.transport.RepositoryTransportFactory;
 import org.gradle.api.internal.file.FileResolver;
@@ -54,21 +57,28 @@ import org.gradle.internal.Cast;
 import org.gradle.internal.action.InstantiatingAction;
 import org.gradle.internal.component.external.model.ModuleComponentArtifactIdentifier;
 import org.gradle.internal.component.external.model.ModuleComponentArtifactMetadata;
+import org.gradle.internal.component.external.model.ModuleDependencyMetadata;
+import org.gradle.internal.component.external.model.MutableModuleComponentResolveMetadata;
 import org.gradle.internal.component.external.model.maven.MutableMavenModuleResolveMetadata;
+import org.gradle.internal.component.model.ComponentOverrideMetadata;
+import org.gradle.internal.hash.Hasher;
+import org.gradle.internal.instantiation.InstantiatorFactory;
 import org.gradle.internal.isolation.IsolatableFactory;
 import org.gradle.internal.reflect.Instantiator;
+import org.gradle.internal.resolve.result.BuildableModuleComponentMetaDataResolveResult;
+import org.gradle.internal.resolve.result.BuildableModuleVersionListingResolveResult;
 import org.gradle.internal.resource.local.FileResourceRepository;
 import org.gradle.internal.resource.local.FileStore;
 import org.gradle.internal.resource.local.LocallyAvailableResourceFinder;
 
+import javax.annotation.Nonnull;
 import java.net.URI;
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Set;
 
-import static org.gradle.api.internal.FeaturePreviews.Feature.GRADLE_METADATA;
-
 public class DefaultMavenArtifactRepository extends AbstractAuthenticationSupportedRepository implements MavenArtifactRepository, ResolutionAwareRepository, PublicationAwareRepository {
     private static final DefaultMavenPomMetadataSource.MavenMetadataValidator NO_OP_VALIDATION_SERVICES = new DefaultMavenPomMetadataSource.MavenMetadataValidator() {
         @Override
@@ -80,17 +90,17 @@ public class DefaultMavenArtifactRepository extends AbstractAuthenticationSuppor
     private final Transformer<String, MavenArtifactRepository> describer;
     private final FileResolver fileResolver;
     private final RepositoryTransportFactory transportFactory;
-    private Object url;
+    private final DefaultUrlArtifactRepository urlArtifactRepository;
     private List<Object> additionalUrls = new ArrayList<Object>();
     private final LocallyAvailableResourceFinder<ModuleComponentArtifactMetadata> locallyAvailableResourceFinder;
     private final FileStore<ModuleComponentArtifactIdentifier> artifactFileStore;
     private final MetaDataParser<MutableMavenModuleResolveMetadata> pomParser;
-    private final ModuleMetadataParser metadataParser;
-    private final ImmutableModuleIdentifierFactory moduleIdentifierFactory;
+    private final GradleModuleMetadataParser metadataParser;
     private final FileStore<String> resourcesFileStore;
     private final FileResourceRepository fileResourceRepository;
     private final MavenMutableModuleMetadataFactory metadataFactory;
     private final IsolatableFactory isolatableFactory;
+    private final ChecksumService checksumService;
     private final MavenMetadataSources metadataSources = new MavenMetadataSources();
     private final InstantiatorFactory instantiatorFactory;
 
@@ -99,18 +109,18 @@ public class DefaultMavenArtifactRepository extends AbstractAuthenticationSuppor
                                           InstantiatorFactory instantiatorFactory,
                                           FileStore<ModuleComponentArtifactIdentifier> artifactFileStore,
                                           MetaDataParser<MutableMavenModuleResolveMetadata> pomParser,
-                                          ModuleMetadataParser metadataParser,
+                                          GradleModuleMetadataParser metadataParser,
                                           AuthenticationContainer authenticationContainer,
-                                          ImmutableModuleIdentifierFactory moduleIdentifierFactory,
                                           FileStore<String> resourcesFileStore,
                                           FileResourceRepository fileResourceRepository,
-                                          FeaturePreviews featurePreviews,
                                           MavenMutableModuleMetadataFactory metadataFactory,
                                           IsolatableFactory isolatableFactory,
-                                          ObjectFactory objectFactory) {
+                                          ObjectFactory objectFactory,
+                                          DefaultUrlArtifactRepository.Factory urlArtifactRepositoryFactory,
+                                          ChecksumService checksumService) {
         this(new DefaultDescriber(), fileResolver, transportFactory, locallyAvailableResourceFinder, instantiatorFactory,
-            artifactFileStore, pomParser, metadataParser, authenticationContainer, moduleIdentifierFactory,
-            resourcesFileStore, fileResourceRepository, featurePreviews, metadataFactory, isolatableFactory, objectFactory);
+            artifactFileStore, pomParser, metadataParser, authenticationContainer,
+            resourcesFileStore, fileResourceRepository, metadataFactory, isolatableFactory, objectFactory, urlArtifactRepositoryFactory, checksumService);
     }
 
     public DefaultMavenArtifactRepository(Transformer<String, MavenArtifactRepository> describer,
@@ -119,29 +129,30 @@ public class DefaultMavenArtifactRepository extends AbstractAuthenticationSuppor
                                           InstantiatorFactory instantiatorFactory,
                                           FileStore<ModuleComponentArtifactIdentifier> artifactFileStore,
                                           MetaDataParser<MutableMavenModuleResolveMetadata> pomParser,
-                                          ModuleMetadataParser metadataParser,
+                                          GradleModuleMetadataParser metadataParser,
                                           AuthenticationContainer authenticationContainer,
-                                          ImmutableModuleIdentifierFactory moduleIdentifierFactory,
                                           FileStore<String> resourcesFileStore,
                                           FileResourceRepository fileResourceRepository,
-                                          FeaturePreviews featurePreviews,
                                           MavenMutableModuleMetadataFactory metadataFactory,
                                           IsolatableFactory isolatableFactory,
-                                          ObjectFactory objectFactory) {
+                                          ObjectFactory objectFactory,
+                                          DefaultUrlArtifactRepository.Factory urlArtifactRepositoryFactory,
+                                          ChecksumService checksumService) {
         super(instantiatorFactory.decorateLenient(), authenticationContainer, objectFactory);
         this.describer = describer;
         this.fileResolver = fileResolver;
+        this.urlArtifactRepository = urlArtifactRepositoryFactory.create("Maven", this::getDisplayName);
         this.transportFactory = transportFactory;
         this.locallyAvailableResourceFinder = locallyAvailableResourceFinder;
         this.artifactFileStore = artifactFileStore;
         this.pomParser = pomParser;
         this.metadataParser = metadataParser;
-        this.moduleIdentifierFactory = moduleIdentifierFactory;
         this.resourcesFileStore = resourcesFileStore;
         this.fileResourceRepository = fileResourceRepository;
         this.metadataFactory = metadataFactory;
         this.isolatableFactory = isolatableFactory;
-        this.metadataSources.setDefaults(featurePreviews);
+        this.checksumService = checksumService;
+        this.metadataSources.setDefaults();
         this.instantiatorFactory = instantiatorFactory;
     }
 
@@ -150,20 +161,35 @@ public class DefaultMavenArtifactRepository extends AbstractAuthenticationSuppor
         return describer.transform(this);
     }
 
+    @Override
     public URI getUrl() {
-        return url == null ? null : fileResolver.resolveUri(url);
+        return urlArtifactRepository.getUrl();
     }
 
+    @Override
     public void setUrl(URI url) {
         invalidateDescriptor();
-        this.url = url;
+        urlArtifactRepository.setUrl(url);
     }
 
+    @Override
     public void setUrl(Object url) {
         invalidateDescriptor();
-        this.url = url;
+        urlArtifactRepository.setUrl(url);
     }
 
+    @Override
+    public void setAllowInsecureProtocol(boolean allowInsecureProtocol) {
+        invalidateDescriptor();
+        urlArtifactRepository.setAllowInsecureProtocol(allowInsecureProtocol);
+    }
+
+    @Override
+    public boolean isAllowInsecureProtocol() {
+        return urlArtifactRepository.isAllowInsecureProtocol();
+    }
+
+    @Override
     public Set<URI> getArtifactUrls() {
         Set<URI> result = new LinkedHashSet<URI>();
         for (Object additionalUrl : additionalUrls) {
@@ -172,6 +198,7 @@ public class DefaultMavenArtifactRepository extends AbstractAuthenticationSuppor
         return result;
     }
 
+    @Override
     public void artifactUrls(Object... urls) {
         invalidateDescriptor();
         additionalUrls.addAll(Lists.newArrayList(urls));
@@ -183,23 +210,26 @@ public class DefaultMavenArtifactRepository extends AbstractAuthenticationSuppor
         setArtifactUrls((Iterable<?>) urls);
     }
 
+    @Override
     public void setArtifactUrls(Iterable<?> urls) {
         invalidateDescriptor();
         additionalUrls = Lists.newArrayList(urls);
     }
 
+    @Override
     public ModuleVersionPublisher createPublisher() {
         return createRealResolver();
     }
 
+    @Override
     public ConfiguredModuleComponentRepository createResolver() {
         return createRealResolver();
     }
 
     @Override
     protected RepositoryDescriptor createDescriptor() {
-        validate();
-        return new MavenRepositoryDescriptor.Builder(getName(), getUrl())
+        URI rootUri = validateUrl();
+        return new MavenRepositoryDescriptor.Builder(getName(), rootUri)
             .setAuthenticated(getConfiguredCredentials() != null)
             .setAuthenticationSchemes(getAuthenticationSchemes())
             .setMetadataSources(metadataSources.asList())
@@ -207,16 +237,26 @@ public class DefaultMavenArtifactRepository extends AbstractAuthenticationSuppor
             .create();
     }
 
-    private void validate() {
-        URI rootUri = getUrl();
-        if (rootUri == null) {
-            throw new InvalidUserDataException("You must specify a URL for a Maven repository.");
+    @Override
+    protected Collection<URI> getRepositoryUrls() {
+        // In a similar way to Ivy, Maven may use other hosts for additional artifacts, but not POMs
+        ImmutableList.Builder<URI> builder = ImmutableList.builder();
+        URI root = getUrl();
+        if (root != null) {
+            builder.add(root);
         }
+        builder.addAll(getArtifactUrls());
+        return builder.build();
+    }
+
+    @Nonnull
+    protected URI validateUrl() {
+        return urlArtifactRepository.validateUrl();
     }
 
     protected MavenResolver createRealResolver() {
-        validate();
-        MavenResolver resolver = createResolver(getUrl());
+        URI rootUrl = validateUrl();
+        MavenResolver resolver = createResolver(rootUrl);
 
         for (URI repoUrl : getArtifactUrls()) {
             resolver.addArtifactLocation(repoUrl);
@@ -231,7 +271,7 @@ public class DefaultMavenArtifactRepository extends AbstractAuthenticationSuppor
         Instantiator injector = createInjectorForMetadataSuppliers(transport, instantiatorFactory, getUrl(), resourcesFileStore);
         InstantiatingAction<ComponentMetadataSupplierDetails> supplier = createComponentMetadataSupplierFactory(injector, isolatableFactory);
         InstantiatingAction<ComponentMetadataListerDetails> lister = createComponentMetadataVersionLister(injector, isolatableFactory);
-        return new MavenResolver(getName(), rootUri, transport, locallyAvailableResourceFinder, artifactFileStore, moduleIdentifierFactory, metadataSources, MavenMetadataArtifactProvider.INSTANCE, mavenMetadataLoader, supplier, lister, injector);
+        return new MavenResolver(getName(), rootUri, transport, locallyAvailableResourceFinder, artifactFileStore, metadataSources, MavenMetadataArtifactProvider.INSTANCE, mavenMetadataLoader, supplier, lister, injector, checksumService);
     }
 
     @Override
@@ -248,13 +288,22 @@ public class DefaultMavenArtifactRepository extends AbstractAuthenticationSuppor
 
     ImmutableMetadataSources createMetadataSources(MavenMetadataLoader mavenMetadataLoader) {
         ImmutableList.Builder<MetadataSource<?>> sources = ImmutableList.builder();
+        // Don't list versions for gradleMetadata if maven-metadata.xml will be checked.
+        boolean listVersionsForGradleMetadata = !metadataSources.mavenPom;
+        MetadataSource<MutableModuleComponentResolveMetadata> gradleModuleMetadataSource =
+            new MavenSnapshotDecoratingSource(
+                new DefaultGradleModuleMetadataSource(getMetadataParser(), metadataFactory, listVersionsForGradleMetadata, checksumService)
+            );
         if (metadataSources.gradleMetadata) {
-            // Don't list versions for gradleMetadata if maven-metadata.xml will be checked.
-            boolean listVersionsForGradleMetadata = !metadataSources.mavenPom;
-            sources.add(new DefaultGradleModuleMetadataSource(getMetadataParser(), metadataFactory, listVersionsForGradleMetadata));
+            sources.add(gradleModuleMetadataSource);
         }
         if (metadataSources.mavenPom) {
-            sources.add(new DefaultMavenPomMetadataSource(MavenMetadataArtifactProvider.INSTANCE, getPomParser(), fileResourceRepository, getMetadataValidationServices(), mavenMetadataLoader));
+            DefaultMavenPomMetadataSource pomMetadataSource = createPomMetadataSource(mavenMetadataLoader, fileResourceRepository);
+            if (metadataSources.ignoreGradleMetadataRedirection) {
+                sources.add(pomMetadataSource);
+            } else {
+                sources.add(new RedirectingGradleMetadataModuleMetadataSource(pomMetadataSource, gradleModuleMetadataSource));
+            }
         }
         if (metadataSources.artifact) {
             sources.add(new DefaultArtifactMetadataSource(metadataFactory));
@@ -262,15 +311,19 @@ public class DefaultMavenArtifactRepository extends AbstractAuthenticationSuppor
         return new DefaultImmutableMetadataSources(sources.build());
     }
 
+    protected DefaultMavenPomMetadataSource createPomMetadataSource(MavenMetadataLoader mavenMetadataLoader, FileResourceRepository fileResourceRepository) {
+        return new DefaultMavenPomMetadataSource(MavenMetadataArtifactProvider.INSTANCE, getPomParser(), fileResourceRepository, getMetadataValidationServices(), mavenMetadataLoader, checksumService);
+    }
+
     protected DefaultMavenPomMetadataSource.MavenMetadataValidator getMetadataValidationServices() {
         return NO_OP_VALIDATION_SERVICES;
     }
 
-    private MetaDataParser<MutableMavenModuleResolveMetadata> getPomParser() {
+    MetaDataParser<MutableMavenModuleResolveMetadata> getPomParser() {
         return pomParser;
     }
 
-    private ModuleMetadataParser getMetadataParser() {
+    private GradleModuleMetadataParser getMetadataParser() {
         return metadataParser;
     }
 
@@ -282,8 +335,8 @@ public class DefaultMavenArtifactRepository extends AbstractAuthenticationSuppor
         return resourcesFileStore;
     }
 
-    RepositoryTransport getTransport(String scheme) {
-        return transportFactory.createTransport(scheme, getName(), getConfiguredAuthentication());
+    public RepositoryTransport getTransport(String scheme) {
+        return transportFactory.createTransport(scheme, getName(), getConfiguredAuthentication(), urlArtifactRepository.createRedirectVerifier());
     }
 
     protected LocallyAvailableResourceFinder<ModuleComponentArtifactMetadata> getLocallyAvailableResourceFinder() {
@@ -314,20 +367,18 @@ public class DefaultMavenArtifactRepository extends AbstractAuthenticationSuppor
         boolean gradleMetadata;
         boolean mavenPom;
         boolean artifact;
+        boolean ignoreGradleMetadataRedirection;
 
-        void setDefaults(FeaturePreviews featurePreviews) {
+        void setDefaults() {
             mavenPom();
-            if (featurePreviews.isFeatureEnabled(GRADLE_METADATA)) {
-                gradleMetadata();
-            } else {
-                artifact();
-            }
+            ignoreGradleMetadataRedirection = false;
         }
 
         void reset() {
             gradleMetadata = false;
             mavenPom = false;
             artifact = false;
+            ignoreGradleMetadataRedirection = false;
         }
 
         /**
@@ -347,6 +398,9 @@ public class DefaultMavenArtifactRepository extends AbstractAuthenticationSuppor
             if (artifact) {
                 list.add("artifact");
             }
+            if (ignoreGradleMetadataRedirection) {
+                list.add("ignoreGradleMetadataRedirection");
+            }
             return list;
         }
 
@@ -364,6 +418,40 @@ public class DefaultMavenArtifactRepository extends AbstractAuthenticationSuppor
         public void artifact() {
             artifact = true;
         }
+
+        @Override
+        public void ignoreGradleMetadataRedirection() {
+            ignoreGradleMetadataRedirection = true;
+        }
     }
 
+    private static class MavenSnapshotDecoratingSource implements MetadataSource<MutableModuleComponentResolveMetadata> {
+        private static final int HASH_ID = 30155977;
+
+        private final MetadataSource<MutableModuleComponentResolveMetadata> delegate;
+
+        private MavenSnapshotDecoratingSource(MetadataSource<MutableModuleComponentResolveMetadata> delegate) {
+            this.delegate = delegate;
+        }
+
+        @Override
+        public MutableModuleComponentResolveMetadata create(String repositoryName, ComponentResolvers componentResolvers, ModuleComponentIdentifier moduleComponentIdentifier, ComponentOverrideMetadata prescribedMetaData, ExternalResourceArtifactResolver artifactResolver, BuildableModuleComponentMetaDataResolveResult result) {
+            MutableModuleComponentResolveMetadata metadata = delegate.create(repositoryName, componentResolvers, moduleComponentIdentifier, prescribedMetaData, artifactResolver, result);
+            if (metadata != null) {
+                return MavenResolver.processMetaData((MutableMavenModuleResolveMetadata) metadata);
+            }
+            return null;
+        }
+
+        @Override
+        public void listModuleVersions(ModuleDependencyMetadata dependency, ModuleIdentifier module, List<ResourcePattern> ivyPatterns, List<ResourcePattern> artifactPatterns, VersionLister versionLister, BuildableModuleVersionListingResolveResult result) {
+            delegate.listModuleVersions(dependency, module, ivyPatterns, artifactPatterns, versionLister, result);
+        }
+
+        @Override
+        public void appendId(Hasher hasher) {
+            hasher.putInt(HASH_ID);
+            delegate.appendId(hasher);
+        }
+    }
 }
