@@ -48,12 +48,14 @@ import org.gradle.internal.component.external.model.MutableModuleComponentResolv
 import org.gradle.internal.component.model.ComponentArtifactMetadata;
 import org.gradle.internal.component.model.ComponentOverrideMetadata;
 import org.gradle.internal.component.model.ComponentResolveMetadata;
+import org.gradle.internal.component.model.ConfigurationMetadata;
 import org.gradle.internal.component.model.DefaultModuleDescriptorArtifactMetadata;
 import org.gradle.internal.component.model.IvyArtifactName;
 import org.gradle.internal.component.model.ModuleDescriptorArtifactMetadata;
-import org.gradle.internal.component.model.ModuleSource;
-import org.gradle.internal.hash.HashUtil;
-import org.gradle.internal.hash.HashValue;
+import org.gradle.internal.component.model.ModuleSources;
+import org.gradle.internal.component.model.WrappedComponentResolveMetadata;
+import org.gradle.internal.hash.ChecksumService;
+import org.gradle.internal.hash.HashCode;
 import org.gradle.internal.hash.Hasher;
 import org.gradle.internal.hash.Hashing;
 import org.gradle.internal.reflect.Instantiator;
@@ -108,6 +110,7 @@ public abstract class ExternalResourceResolver<T extends ModuleComponentResolveM
     private final InstantiatingAction<ComponentMetadataSupplierDetails> componentMetadataSupplierFactory;
     private final InstantiatingAction<ComponentMetadataListerDetails> providedVersionLister;
     private final Instantiator injector;
+    private final ChecksumService checksumService;
 
     private String id;
     private ExternalResourceArtifactResolver cachedArtifactResolver;
@@ -122,7 +125,8 @@ public abstract class ExternalResourceResolver<T extends ModuleComponentResolveM
                                        MetadataArtifactProvider metadataArtifactProvider,
                                        @Nullable InstantiatingAction<ComponentMetadataSupplierDetails> componentMetadataSupplierFactory,
                                        @Nullable InstantiatingAction<ComponentMetadataListerDetails> providedVersionLister,
-                                       Instantiator injector) {
+                                       Instantiator injector,
+                                       ChecksumService checksumService) {
         this.name = name;
         this.local = local;
         this.cachingResourceAccessor = cachingResourceAccessor;
@@ -134,6 +138,7 @@ public abstract class ExternalResourceResolver<T extends ModuleComponentResolveM
         this.componentMetadataSupplierFactory = componentMetadataSupplierFactory;
         this.providedVersionLister = providedVersionLister;
         this.injector = injector;
+        this.checksumService = checksumService;
     }
 
     @Override
@@ -246,7 +251,6 @@ public abstract class ExternalResourceResolver<T extends ModuleComponentResolveM
         for (MetadataSource<?> source : metadataSources.sources()) {
             MutableModuleComponentResolveMetadata value = source.create(name, componentResolvers, moduleVersionIdentifier, prescribedMetaData, artifactResolver, result);
             if (value != null) {
-                value.setSource(artifactResolver.getSource());
                 result.resolved(value.asImmutable());
                 return;
             }
@@ -260,7 +264,7 @@ public abstract class ExternalResourceResolver<T extends ModuleComponentResolveM
 
     protected Set<ModuleComponentArtifactMetadata> findOptionalArtifacts(ModuleComponentResolveMetadata module, String type, String classifier) {
         ModuleComponentArtifactMetadata artifact = module.artifact(type, "jar", classifier);
-        if (createArtifactResolver(module.getSource()).artifactExists(artifact, new DefaultResourceAwareResolveResult())) {
+        if (createArtifactResolver(module.getSources()).artifactExists(artifact, new DefaultResourceAwareResolveResult())) {
             return ImmutableSet.of(artifact);
         }
         return Collections.emptySet();
@@ -284,7 +288,7 @@ public abstract class ExternalResourceResolver<T extends ModuleComponentResolveM
         return new DefaultExternalResourceArtifactResolver(repository, locallyAvailableResourceFinder, ivyPatterns, artifactPatterns, artifactFileStore, cachingResourceAccessor);
     }
 
-    protected ExternalResourceArtifactResolver createArtifactResolver(ModuleSource moduleSource) {
+    protected ExternalResourceArtifactResolver createArtifactResolver(ModuleSources moduleSources) {
         return createArtifactResolver();
     }
 
@@ -312,18 +316,35 @@ public abstract class ExternalResourceResolver<T extends ModuleComponentResolveM
 
     private void put(File src, ExternalResourceName destination) {
         repository.withProgressLogging().resource(destination).put(new FileReadableContent(src));
-        putChecksum(src, destination);
+        publishChecksums(destination, src);
     }
 
-    private void putChecksum(File source, ExternalResourceName destination) {
-        byte[] checksumFile = createChecksumFile(source, "SHA1", 40);
-        ExternalResourceName checksumDestination = destination.append(".sha1");
-        repository.resource(checksumDestination).put(new ByteArrayReadableContent(checksumFile));
+    private void publishChecksums(ExternalResourceName destination, File content) {
+        publishChecksum(destination, content, "sha1", 40);
+
+        if (!ExternalResourceResolver.disableExtraChecksums()) {
+            publishPossiblyUnsupportedChecksum(destination, content, "sha-256", 64);
+            publishPossiblyUnsupportedChecksum(destination, content, "sha-512", 128);
+        }
+    }
+
+    private void publishPossiblyUnsupportedChecksum(ExternalResourceName destination, File content, String algorithm, int length) {
+        try {
+            publishChecksum(destination, content, algorithm, length);
+        } catch (Exception ex) {
+            LOGGER.warn("Cannot upload checksum for " + content.getName() + ". Remote repository doesn't support " + algorithm + ". Error: " + ex.getMessage());
+        }
+    }
+
+    private void publishChecksum(ExternalResourceName destination, File content, String algorithm, int length) {
+        byte[] checksum = createChecksumFile(content, algorithm.toUpperCase(), length);
+        ExternalResourceName checksumDestination = destination.append("." + algorithm.replaceAll("-", ""));
+        repository.resource(checksumDestination).put(new ByteArrayReadableContent(checksum));
     }
 
     private byte[] createChecksumFile(File src, String algorithm, int checksumLength) {
-        HashValue hash = HashUtil.createHash(src, algorithm);
-        String formattedHashString = hash.asZeroPaddedHexString(checksumLength);
+        HashCode hash = checksumService.hash(src, algorithm);
+        String formattedHashString = hash.toString();
         try {
             return formattedHashString.getBytes("US-ASCII");
         } catch (UnsupportedEncodingException e) {
@@ -390,12 +411,19 @@ public abstract class ExternalResourceResolver<T extends ModuleComponentResolveM
         }
 
         @Override
-        public void resolveArtifacts(ComponentResolveMetadata component, BuildableComponentArtifactsResolveResult result) {
-            T moduleMetaData = getSupportedMetadataType().cast(component);
-            resolveModuleArtifacts(moduleMetaData, result);
+        public void resolveArtifacts(ComponentResolveMetadata component, ConfigurationMetadata variant, BuildableComponentArtifactsResolveResult result) {
+            T moduleMetaData = getSupportedMetadataType().cast(unwrap(component));
+            resolveModuleArtifacts(moduleMetaData, variant, result);
         }
 
-        protected abstract void resolveModuleArtifacts(T module, BuildableComponentArtifactsResolveResult result);
+        private ComponentResolveMetadata unwrap(ComponentResolveMetadata original) {
+            if (original instanceof WrappedComponentResolveMetadata) {
+                return ((WrappedComponentResolveMetadata) original).unwrap();
+            }
+            return original;
+        }
+
+        protected abstract void resolveModuleArtifacts(T module, ConfigurationMetadata variant, BuildableComponentArtifactsResolveResult result);
 
         protected abstract void resolveMetaDataArtifacts(T module, BuildableArtifactSetResolveResult result);
 
@@ -425,7 +453,7 @@ public abstract class ExternalResourceResolver<T extends ModuleComponentResolveM
         }
 
         @Override
-        public void resolveArtifact(ComponentArtifactMetadata artifact, ModuleSource moduleSource, BuildableArtifactResolveResult result) {
+        public void resolveArtifact(ComponentArtifactMetadata artifact, ModuleSources moduleSources, BuildableArtifactResolveResult result) {
 
         }
 
@@ -458,8 +486,8 @@ public abstract class ExternalResourceResolver<T extends ModuleComponentResolveM
         }
 
         @Override
-        public void resolveArtifacts(ComponentResolveMetadata component, BuildableComponentArtifactsResolveResult result) {
-            super.resolveArtifacts(component, result);
+        public void resolveArtifacts(ComponentResolveMetadata component, ConfigurationMetadata variant, BuildableComponentArtifactsResolveResult result) {
+            super.resolveArtifacts(component, variant, result);
             checkArtifactsResolved(component, "artifacts", result);
         }
 
@@ -476,9 +504,9 @@ public abstract class ExternalResourceResolver<T extends ModuleComponentResolveM
         }
 
         @Override
-        public void resolveArtifact(ComponentArtifactMetadata artifact, ModuleSource moduleSource, BuildableArtifactResolveResult result) {
+        public void resolveArtifact(ComponentArtifactMetadata artifact, ModuleSources moduleSources, BuildableArtifactResolveResult result) {
             try {
-                ExternalResourceArtifactResolver resolver = createArtifactResolver(moduleSource);
+                ExternalResourceArtifactResolver resolver = createArtifactResolver(moduleSources);
                 ModuleComponentArtifactMetadata moduleArtifact = (ModuleComponentArtifactMetadata) artifact;
                 LocallyAvailableExternalResource artifactResource = resolver.resolveArtifact(moduleArtifact, result);
                 if (artifactResource == null) {
@@ -568,4 +596,9 @@ public abstract class ExternalResourceResolver<T extends ModuleComponentResolveM
             result.listed(versions);
         }
     }
+
+    public static boolean disableExtraChecksums() {
+        return Boolean.getBoolean("org.gradle.internal.publish.checksums.insecure");
+    }
+
 }
