@@ -27,7 +27,7 @@ import java.util.Collection;
 import java.util.List;
 
 /**
- * Represents dependency information as stored in an external descriptor file.
+ * Represents dependency information as stored in an external descriptor file (POM or IVY.XML).
  * This information is able to be transformed into a `ModuleDependencyMetadata` instance.
  */
 public abstract class ExternalDependencyDescriptor {
@@ -36,6 +36,8 @@ public abstract class ExternalDependencyDescriptor {
 
     public abstract boolean isOptional();
 
+    public abstract boolean isConstraint();
+
     public abstract boolean isChanging();
 
     public abstract boolean isTransitive();
