@@ -20,6 +20,8 @@ import org.gradle.api.Action;
 import org.gradle.api.artifacts.Configuration;
 import org.gradle.api.artifacts.ConfigurationContainer;
 import org.gradle.api.artifacts.dsl.DependencyLockingHandler;
+import org.gradle.api.artifacts.dsl.LockMode;
+import org.gradle.api.internal.artifacts.dsl.dependencies.DependencyLockingProvider;
 
 public class DefaultDependencyLockingHandler implements DependencyLockingHandler {
 
@@ -40,9 +42,11 @@ public class DefaultDependencyLockingHandler implements DependencyLockingHandler
 
 
     private final ConfigurationContainer configurationContainer;
+    private final DependencyLockingProvider dependencyLockingProvider;
 
-    public DefaultDependencyLockingHandler(ConfigurationContainer configurationContainer) {
+    public DefaultDependencyLockingHandler(ConfigurationContainer configurationContainer, DependencyLockingProvider dependencyLockingProvider) {
         this.configurationContainer = configurationContainer;
+        this.dependencyLockingProvider = dependencyLockingProvider;
     }
 
     @Override
@@ -54,4 +58,14 @@ public class DefaultDependencyLockingHandler implements DependencyLockingHandler
     public void unlockAllConfigurations() {
         configurationContainer.all(DEACTIVATE_LOCKING);
     }
+
+    @Override
+    public LockMode getLockMode() {
+        return dependencyLockingProvider.getLockMode();
+    }
+
+    @Override
+    public void setLockMode(LockMode mode) {
+        dependencyLockingProvider.setLockMode(mode);
+    }
 }
