@@ -73,6 +73,7 @@ import java.util.List;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Set;
+import java.util.stream.Collectors;
 
 import static org.gradle.api.internal.artifacts.BaseRepositoryFactory.PLUGIN_PORTAL_OVERRIDE_URL_PROPERTY;
 import static org.gradle.integtests.fixtures.RepoScriptBlockUtil.gradlePluginRepositoryMirrorUrl;
@@ -98,10 +99,15 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
 
     protected final static Set<String> PROPAGATED_SYSTEM_PROPERTIES = Sets.newHashSet();
 
+    // TODO - don't use statics to communicate between the test runner and executer
     public static void propagateSystemProperty(String name) {
         PROPAGATED_SYSTEM_PROPERTIES.add(name);
     }
 
+    public static void doNotPropagateSystemProperty(String name) {
+        PROPAGATED_SYSTEM_PROPERTIES.remove(name);
+    }
+
     private static final String DEBUG_SYSPROP = "org.gradle.integtest.debug";
     private static final String LAUNCHER_DEBUG_SYSPROP = "org.gradle.integtest.launcher.debug";
     private static final String PROFILE_SYSPROP = "org.gradle.integtest.profile";
@@ -150,7 +156,8 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
     private boolean showStacktrace = true;
     private boolean renderWelcomeMessage;
 
-    private int expectedDeprecationWarnings;
+    private int expectedGenericDeprecationWarnings;
+    private final List<String> expectedDeprecationWarnings = new ArrayList<>();
     private boolean eagerClassLoaderCreationChecksOn = true;
     private boolean stackTraceChecksOn = true;
 
@@ -219,7 +226,8 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         commandLineJvmOpts.clear();
         buildJvmOpts.clear();
         useOnlyRequestedJvmOpts = false;
-        expectedDeprecationWarnings = 0;
+        expectedGenericDeprecationWarnings = 0;
+        expectedDeprecationWarnings.clear();
         stackTraceChecksOn = true;
         renderWelcomeMessage = false;
         debug = Boolean.getBoolean(DEBUG_SYSPROP);
@@ -344,9 +352,10 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         }
         executer.noExtraLogging();
 
-        if (expectedDeprecationWarnings > 0) {
-            executer.expectDeprecationWarnings(expectedDeprecationWarnings);
+        if (expectedGenericDeprecationWarnings > 0) {
+            executer.expectDeprecationWarnings(expectedGenericDeprecationWarnings);
         }
+        expectedDeprecationWarnings.forEach(executer::expectDeprecationWarning);
         if (!eagerClassLoaderCreationChecksOn) {
             executer.withEagerClassLoaderCreationCheckDisabled();
         }
@@ -530,6 +539,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
             buildJvmOpts.add("-Xmx512m");
         }
         if (JVM_VERSION_DETECTOR.getJavaVersion(Jvm.forHome(getJavaHome())).compareTo(JavaVersion.VERSION_1_8) < 0) {
+            // Although Gradle isn't supported on earlier versions, some tests do run it using Java 6 and 7 to verify it behaves well in this case
             buildJvmOpts.add("-XX:MaxPermSize=320m");
         } else {
             buildJvmOpts.add("-XX:MaxMetaspaceSize=512m");
@@ -1132,9 +1142,10 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
 
     protected Action<ExecutionResult> getResultAssertion() {
         return new Action<ExecutionResult>() {
-            int expectedDeprecationWarnings = AbstractGradleExecuter.this.expectedDeprecationWarnings;
-            boolean expectStackTraces = !AbstractGradleExecuter.this.stackTraceChecksOn;
-            boolean checkDeprecations = AbstractGradleExecuter.this.checkDeprecations;
+            private int expectedGenericDeprecationWarnings = AbstractGradleExecuter.this.expectedGenericDeprecationWarnings;
+            private final List<String> expectedDeprecationWarnings = new ArrayList<>(AbstractGradleExecuter.this.expectedDeprecationWarnings);
+            private final boolean expectStackTraces = !AbstractGradleExecuter.this.stackTraceChecksOn;
+            private final boolean checkDeprecations = AbstractGradleExecuter.this.checkDeprecations;
 
             @Override
             public void execute(ExecutionResult executionResult) {
@@ -1155,8 +1166,14 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
 
                 validate(error, "Standard error");
 
-                if (expectedDeprecationWarnings > 0) {
-                    throw new AssertionError(String.format("Expected %d more deprecation warnings", expectedDeprecationWarnings));
+                if (!expectedDeprecationWarnings.isEmpty()) {
+                    throw new AssertionError(String.format("Expected the following deprecation warnings:%n%s",
+                        expectedDeprecationWarnings.stream()
+                            .map(warning -> " - " + warning)
+                            .collect(Collectors.joining("\n"))));
+                }
+                if (expectedGenericDeprecationWarnings > 0) {
+                    throw new AssertionError(String.format("Expected %d more deprecation warnings", expectedGenericDeprecationWarnings));
                 }
             }
 
@@ -1203,16 +1220,18 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
                         i++;
                     } else if (isDeprecationMessageInHelpDescription(line)) {
                         i++;
+                    } else if (expectedDeprecationWarnings.remove(line)) {
+                        // Deprecation warning is expected
+                        i++;
+                        i = skipStackTrace(lines, i);
                     } else if (line.matches(".*\\s+deprecated.*")) {
-                        if (checkDeprecations && expectedDeprecationWarnings <= 0) {
+                        if (checkDeprecations && expectedGenericDeprecationWarnings <= 0) {
                             throw new AssertionError(String.format("%s line %d contains a deprecation warning: %s%n=====%n%s%n=====%n", displayName, i + 1, line, output));
                         }
-                        expectedDeprecationWarnings--;
+                        expectedGenericDeprecationWarnings--;
                         // skip over stack trace
                         i++;
-                        while (i < lines.size() && STACK_TRACE_ELEMENT.matcher(lines.get(i)).matches()) {
-                            i++;
-                        }
+                        i = skipStackTrace(lines, i);
                     } else if (!expectStackTraces && !insideVariantDescriptionBlock && STACK_TRACE_ELEMENT.matcher(line).matches() && i < lines.size() - 1 && STACK_TRACE_ELEMENT.matcher(lines.get(i + 1)).matches()) {
                         // 2 or more lines that look like stack trace elements
                         throw new AssertionError(String.format("%s line %d contains an unexpected stack trace: %s%n=====%n%s%n=====%n", displayName, i + 1, line, output));
@@ -1222,6 +1241,13 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
                 }
             }
 
+            private int skipStackTrace(List<String> lines, int i) {
+                while (i < lines.size() && STACK_TRACE_ELEMENT.matcher(lines.get(i)).matches()) {
+                    i++;
+                }
+                return i;
+            }
+
             private boolean isDeprecationMessageInHelpDescription(String s) {
                 return s.matches(".*\\[deprecated.*]");
             }
@@ -1235,9 +1261,15 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
 
     @Override
     public GradleExecuter expectDeprecationWarnings(int count) {
-        Preconditions.checkState(expectedDeprecationWarnings == 0, "expected deprecation count is already set for this execution");
+        Preconditions.checkState(expectedGenericDeprecationWarnings == 0, "expected deprecation count is already set for this execution");
         Preconditions.checkArgument(count > 0, "expected deprecation count must be positive");
-        expectedDeprecationWarnings = count;
+        expectedGenericDeprecationWarnings = count;
+        return this;
+    }
+
+    @Override
+    public GradleExecuter expectDeprecationWarning(String warning) {
+        expectedDeprecationWarnings.add(warning);
         return this;
     }
 
