@@ -60,6 +60,7 @@ import org.gradle.platform.base.ComponentType;
 import org.gradle.platform.base.DependencySpec;
 import org.gradle.platform.base.TypeBuilder;
 import org.gradle.platform.base.internal.BinarySpecInternal;
+import org.gradle.util.DeprecationLogger;
 
 import java.io.File;
 import java.util.Collections;
@@ -77,10 +78,12 @@ import static org.gradle.util.CollectionUtils.first;
  * @since 3.4
  */
 @Incubating
+@Deprecated
 public class JavaLanguagePlugin implements Plugin<Project> {
 
     @Override
     public void apply(Project project) {
+        DeprecationLogger.nagUserOfDeprecatedPlugin("java-lang", 6, "upgrading_jvm_plugins");
         project.getPluginManager().apply(ComponentModelBasePlugin.class);
         project.getPluginManager().apply(JvmResourcesPlugin.class);
     }
@@ -168,7 +171,7 @@ public class JavaLanguagePlugin implements Plugin<Project> {
                 assembly.builtBy(compile);
 
                 compile.setDescription("Compiles " + javaSourceSet + ".");
-                compile.setDestinationDir(conventionalCompilationOutputDirFor(assembly));
+                compile.getDestinationDirectory().set(conventionalCompilationOutputDirFor(assembly));
                 compile.dependsOn(javaSourceSet);
                 compile.setSource(javaSourceSet.getSource());
 
