@@ -16,7 +16,10 @@
 
 package org.gradle.language.jvm.plugins;
 
-import org.gradle.api.*;
+import org.gradle.api.DefaultTask;
+import org.gradle.api.Plugin;
+import org.gradle.api.Project;
+import org.gradle.api.Task;
 import org.gradle.internal.service.ServiceRegistry;
 import org.gradle.jvm.internal.JvmAssembly;
 import org.gradle.jvm.internal.WithJvmAssembly;
@@ -31,8 +34,9 @@ import org.gradle.language.jvm.tasks.ProcessResources;
 import org.gradle.model.Mutate;
 import org.gradle.model.RuleSource;
 import org.gradle.platform.base.BinarySpec;
-import org.gradle.platform.base.LanguageType;
-import org.gradle.platform.base.LanguageTypeBuilder;
+import org.gradle.platform.base.ComponentType;
+import org.gradle.platform.base.TypeBuilder;
+import org.gradle.util.DeprecationLogger;
 
 import java.util.Collections;
 import java.util.Map;
@@ -43,18 +47,19 @@ import static org.gradle.util.CollectionUtils.first;
  * Plugin for packaging JVM resources. Applies the {@link org.gradle.language.base.plugins.ComponentModelBasePlugin}. Registers "resources" language support with the {@link
  * org.gradle.language.jvm.JvmResourceSet}.
  */
-@Incubating
+@Deprecated
 public class JvmResourcesPlugin implements Plugin<Project> {
 
+    @Override
     public void apply(final Project project) {
+        DeprecationLogger.nagUserOfDeprecatedPlugin("jvm-resources", 6, "upgrading_jvm_plugins");
         project.getPluginManager().apply(ComponentModelBasePlugin.class);
     }
 
     @SuppressWarnings("UnusedDeclaration")
     static class Rules extends RuleSource {
-        @LanguageType
-        void registerLanguage(LanguageTypeBuilder<JvmResourceSet> builder) {
-            builder.setLanguageName("resources");
+        @ComponentType
+        void registerLanguage(TypeBuilder<JvmResourceSet> builder) {
             builder.defaultImplementation(DefaultJvmResourceLanguageSourceSet.class);
         }
 
@@ -65,34 +70,46 @@ public class JvmResourcesPlugin implements Plugin<Project> {
     }
 
     private static class JvmResources implements LanguageTransform<JvmResourceSet, org.gradle.jvm.JvmResources> {
+        @Override
+        public String getLanguageName() {
+            return "resources";
+        }
+
+        @Override
         public Class<JvmResourceSet> getSourceSetType() {
             return JvmResourceSet.class;
         }
 
+        @Override
         public Map<String, Class<?>> getBinaryTools() {
             return Collections.emptyMap();
         }
 
+        @Override
         public Class<org.gradle.jvm.JvmResources> getOutputType() {
             return org.gradle.jvm.JvmResources.class;
         }
 
+        @Override
         public SourceTransformTaskConfig getTransformTask() {
             return new SourceTransformTaskConfig() {
+                @Override
                 public String getTaskPrefix() {
                     return "process";
                 }
 
+                @Override
                 public Class<? extends DefaultTask> getTaskType() {
                     return ProcessResources.class;
                 }
 
+                @Override
                 public void configureTask(Task task, BinarySpec binary, LanguageSourceSet sourceSet, ServiceRegistry serviceRegistry) {
                     ProcessResources resourcesTask = (ProcessResources) task;
                     JvmResourceSet resourceSet = (JvmResourceSet) sourceSet;
                     resourcesTask.from(resourceSet.getSource());
 
-                    // The first directory is the one created by JarBinaryRules
+                    // The first directory is the one created by JvmComponentPlugin.configureJvmBinaries()
                     // to be used as the default output directory for processed resources
                     JvmAssembly assembly = ((WithJvmAssembly) binary).getAssembly();
                     resourcesTask.setDestinationDir(first(assembly.getResourceDirectories()));
@@ -101,6 +118,7 @@ public class JvmResourcesPlugin implements Plugin<Project> {
                 }
             };
         }
+        @Override
         public boolean applyToBinary(BinarySpec binary) {
             return binary instanceof WithJvmAssembly;
         }
