@@ -16,7 +16,11 @@
 
 package org.gradle.language.scala.plugins;
 
-import org.gradle.api.*;
+import org.gradle.api.DefaultTask;
+import org.gradle.api.Incubating;
+import org.gradle.api.Plugin;
+import org.gradle.api.Project;
+import org.gradle.api.Task;
 import org.gradle.api.file.FileCollection;
 import org.gradle.internal.service.ServiceRegistry;
 import org.gradle.jvm.JvmByteCode;
@@ -40,10 +44,10 @@ import org.gradle.model.Model;
 import org.gradle.model.Mutate;
 import org.gradle.model.RuleSource;
 import org.gradle.platform.base.BinarySpec;
-import org.gradle.platform.base.LanguageType;
-import org.gradle.platform.base.LanguageTypeBuilder;
+import org.gradle.platform.base.ComponentType;
+import org.gradle.platform.base.TypeBuilder;
+import org.gradle.util.DeprecationLogger;
 
-import java.io.File;
 import java.util.Collections;
 import java.util.Map;
 
@@ -55,9 +59,12 @@ import static org.gradle.util.CollectionUtils.single;
  * Registers "scala" language support with the {@link org.gradle.language.scala.ScalaLanguageSourceSet}.
  */
 @Incubating
+@Deprecated
 public class ScalaLanguagePlugin implements Plugin<Project> {
 
+    @Override
     public void apply(Project project) {
+        DeprecationLogger.nagUserOfDeprecatedPlugin("scala-lang", 6, "upgrading_jvm_plugins");
         project.getPluginManager().apply(ComponentModelBasePlugin.class);
         project.getPluginManager().apply(JvmResourcesPlugin.class);
     }
@@ -70,9 +77,8 @@ public class ScalaLanguagePlugin implements Plugin<Project> {
             return serviceRegistry.get(ScalaToolChain.class);
         }
 
-        @LanguageType
-        void registerLanguage(LanguageTypeBuilder<ScalaLanguageSourceSet> builder) {
-            builder.setLanguageName("scala");
+        @ComponentType
+        void registerLanguage(TypeBuilder<ScalaLanguageSourceSet> builder) {
             builder.defaultImplementation(DefaultScalaLanguageSourceSet.class);
         }
 
@@ -84,20 +90,30 @@ public class ScalaLanguagePlugin implements Plugin<Project> {
 
 
     private static class Scala implements LanguageTransform<ScalaLanguageSourceSet, JvmByteCode> {
+        @Override
+        public String getLanguageName() {
+            return "scala";
+        }
+
+        @Override
         public Class<ScalaLanguageSourceSet> getSourceSetType() {
             return ScalaLanguageSourceSet.class;
         }
 
+        @Override
         public Map<String, Class<?>> getBinaryTools() {
             return Collections.emptyMap();
         }
 
+        @Override
         public Class<JvmByteCode> getOutputType() {
             return JvmByteCode.class;
         }
 
+        @Override
         public JointCompileTaskConfig getTransformTask() {
             return new JointCompileTaskConfig() {
+                @Override
                 public String getTaskPrefix() {
                     return "compile";
                 }
@@ -107,13 +123,15 @@ public class ScalaLanguagePlugin implements Plugin<Project> {
                     return candidate instanceof ScalaLanguageSourceSet || candidate instanceof JavaSourceSet;
                 }
 
+                @Override
                 public Class<? extends DefaultTask> getTaskType() {
                     return PlatformScalaCompile.class;
                 }
 
+                @Override
                 public void configureTask(Task task, BinarySpec binarySpec, LanguageSourceSet sourceSet, ServiceRegistry serviceRegistry) {
                     PlatformScalaCompile compile = (PlatformScalaCompile) task;
-                    configureScalaTask(compile, ((WithJvmAssembly) binarySpec).getAssembly(), String.format("Compiles %s.", sourceSet));
+                    configureScalaTask(compile, ((WithJvmAssembly) binarySpec).getAssembly(), "Compiles " + sourceSet + ".");
                     addSourceSetToCompile(compile, sourceSet);
                     addSourceSetClasspath(compile, (ScalaLanguageSourceSet) sourceSet);
                 }
@@ -128,9 +146,11 @@ public class ScalaLanguagePlugin implements Plugin<Project> {
                     assembly.builtBy(compile);
 
                     compile.setDescription(description);
-                    compile.setDestinationDir(single(assembly.getClassDirectories()));
-                    File analysisFile = new File(compile.getProject().getBuildDir(), String.format("tmp/scala/compilerAnalysis/%s.analysis", compile.getName()));
-                    compile.getScalaCompileOptions().getIncrementalOptions().setAnalysisFile(analysisFile);
+                    compile.getDestinationDirectory().set(single(assembly.getClassDirectories()));
+
+                    compile.getScalaCompileOptions().getIncrementalOptions().getAnalysisFile().set(
+                        compile.getProject().getLayout().getBuildDirectory().file("tmp/scala/compilerAnalysis/" + compile.getName() + ".analysis")
+                    );
 
                     JavaPlatform javaPlatform = assembly.getTargetPlatform();
                     String targetCompatibility = javaPlatform.getTargetCompatibility().toString();
@@ -140,8 +160,7 @@ public class ScalaLanguagePlugin implements Plugin<Project> {
                     if (assembly instanceof ScalaJvmAssembly) {
                         compile.setPlatform(((ScalaJvmAssembly) assembly).getScalaPlatform());
                     } else {
-                        // TODO:DAZ Put the default scala platform somewhere else, or enforce that we always have a `ScalaJvmAssembly`
-                        compile.setPlatform(new DefaultScalaPlatform("2.10.4"));
+                        compile.setPlatform(new DefaultScalaPlatform());
                     }
                 }
 
@@ -158,6 +177,7 @@ public class ScalaLanguagePlugin implements Plugin<Project> {
             };
         }
 
+        @Override
         public boolean applyToBinary(BinarySpec binary) {
             return binary instanceof WithJvmAssembly;
         }
