@@ -17,7 +17,6 @@
 package org.gradle.api.internal.tasks;
 
 import org.gradle.api.Project;
-import org.gradle.internal.service.ServiceRegistry;
 
 import javax.annotation.Nullable;
 
@@ -34,5 +33,5 @@ public interface WorkNodeAction {
     /**
      * Run the action, throwing any failure.
      */
-    void run(ServiceRegistry registry);
+    void run(NodeExecutionContext context);
 }
