@@ -71,6 +71,7 @@ public class DefaultCacheFactory implements CacheFactory, Closeable {
         }
     }
 
+    @Override
     public void close() {
         lock.lock();
         try {
@@ -113,7 +114,7 @@ public class DefaultCacheFactory implements CacheFactory, Closeable {
         private final CacheBuilder.LockTarget lockTarget;
         private final LockOptions lockOptions;
         private final ReferencablePersistentCache cache;
-        private final Set<ReferenceTrackingCache> references = new HashSet<ReferenceTrackingCache>();
+        private final Set<ReferenceTrackingCache> references = new HashSet<>();
 
         DirCacheReference(ReferencablePersistentCache cache, Map<String, ?> properties, CacheBuilder.LockTarget lockTarget, LockOptions lockOptions) {
             this.cache = cache;
@@ -138,6 +139,7 @@ public class DefaultCacheFactory implements CacheFactory, Closeable {
             }
         }
 
+        @Override
         public void close() {
             onClose(cache);
             dirCaches.values().remove(this);
