@@ -21,8 +21,11 @@ import groovy.lang.MissingPropertyException;
 import groovy.xml.MarkupBuilder;
 import org.gradle.api.tasks.Input;
 import org.gradle.api.tasks.InputFiles;
+import org.gradle.api.tasks.Internal;
 import org.gradle.api.tasks.Optional;
 import org.gradle.api.tasks.OutputDirectory;
+import org.gradle.api.tasks.PathSensitive;
+import org.gradle.api.tasks.PathSensitivity;
 import org.gradle.api.tasks.testing.TestFrameworkOptions;
 import org.gradle.internal.ErroringAction;
 import org.gradle.internal.IoActions;
@@ -96,6 +99,7 @@ public class TestNGOptions extends TestFrameworkOptions {
         }
     }
 
+    @Internal
     protected File getProjectDir() {
         return projectDir;
     }
@@ -213,6 +217,7 @@ public class TestNGOptions extends TestFrameworkOptions {
     /**
      * Option for what to do for other tests that use a configuration step when that step fails. Can be "skip" or "continue", defaults to "skip".
      */
+    @Internal
     public String getConfigFailurePolicy() {
         return configFailurePolicy;
     }
@@ -237,6 +242,7 @@ public class TestNGOptions extends TestFrameworkOptions {
      * }
      * </pre>
      */
+    @Internal
     public Set<String> getListeners() {
         return listeners;
     }
@@ -253,6 +259,7 @@ public class TestNGOptions extends TestFrameworkOptions {
      * If not present, parallel mode will not be selected
      */
     @Nullable
+    @Internal
     public String getParallel() {
         return parallel;
     }
@@ -264,6 +271,7 @@ public class TestNGOptions extends TestFrameworkOptions {
     /**
      * The number of threads to use for this run. Ignored unless the parallel mode is also specified
      */
+    @Internal
     public int getThreadCount() {
         return threadCount;
     }
@@ -272,6 +280,7 @@ public class TestNGOptions extends TestFrameworkOptions {
         this.threadCount = threadCount;
     }
 
+    @Internal
     public boolean getUseDefaultListeners() {
         return useDefaultListeners;
     }
@@ -299,6 +308,7 @@ public class TestNGOptions extends TestFrameworkOptions {
      * Please refer to the documentation of your version of TestNG what are the default listeners. At the moment of writing this documentation, the default listeners are a set of reporters that
      * generate: TestNG variant of HTML results, TestNG variant of XML results in JUnit format, emailable HTML test report, XML results in TestNG format.
      */
+    @Internal
     public boolean isUseDefaultListeners() {
         return useDefaultListeners;
     }
@@ -310,6 +320,7 @@ public class TestNGOptions extends TestFrameworkOptions {
     /**
      * Sets the default name of the test suite, if one is not specified in a suite XML file or in the source code.
      */
+    @Internal
     public String getSuiteName() {
         return suiteName;
     }
@@ -321,6 +332,7 @@ public class TestNGOptions extends TestFrameworkOptions {
     /**
      * Sets the default name of the test, if one is not specified in a suite XML file or in the source code.
      */
+    @Internal
     public String getTestName() {
         return testName;
     }
@@ -335,6 +347,7 @@ public class TestNGOptions extends TestFrameworkOptions {
      * Note: The suiteXmlFiles can be used in conjunction with the suiteXmlBuilder.
      */
     @InputFiles
+    @PathSensitive(PathSensitivity.NONE)
     public List<File> getSuiteXmlFiles() {
         return suiteXmlFiles;
     }
@@ -343,6 +356,7 @@ public class TestNGOptions extends TestFrameworkOptions {
         this.suiteXmlFiles = suiteXmlFiles;
     }
 
+    @Internal
     public boolean getPreserveOrder() {
         return preserveOrder;
     }
@@ -355,6 +369,7 @@ public class TestNGOptions extends TestFrameworkOptions {
      *
      * If not present, the order will not be preserved.
      */
+    @Internal
     public boolean isPreserveOrder() {
         return preserveOrder;
     }
@@ -363,6 +378,7 @@ public class TestNGOptions extends TestFrameworkOptions {
         this.preserveOrder = preserveOrder;
     }
 
+    @Internal
     public boolean getGroupByInstances() {
         return groupByInstances;
     }
@@ -375,6 +391,7 @@ public class TestNGOptions extends TestFrameworkOptions {
      *
      * If not present, the tests will not be grouped by instances.
      */
+    @Internal
     public boolean isGroupByInstances() {
         return groupByInstances;
     }
@@ -390,10 +407,11 @@ public class TestNGOptions extends TestFrameworkOptions {
      */
     @Input
     @Optional
-    private String getSuiteXml() {
+    protected String getSuiteXml() {
         return suiteXmlWriter == null ? null : suiteXmlWriter.toString();
     }
 
+    @Internal
     public StringWriter getSuiteXmlWriter() {
         return suiteXmlWriter;
     }
@@ -402,6 +420,7 @@ public class TestNGOptions extends TestFrameworkOptions {
         this.suiteXmlWriter = suiteXmlWriter;
     }
 
+    @Internal
     public MarkupBuilder getSuiteXmlBuilder() {
         return suiteXmlBuilder;
     }
